@@ -8,6 +8,7 @@
 # Standard packages.
 import re
 import datetime
+import warnings
 from collections import defaultdict
 from collections import namedtuple
 
@@ -170,7 +171,7 @@ class Worksheet(xmlwriter.XMLwriter):
 
         self.repeat_row_range = ''
         self.repeat_col_range = ''
-        self.print_area = ''
+        self.print_area_range = ''
 
         self.page_order = 0
         self.black_white = 0
@@ -262,6 +263,8 @@ class Worksheet(xmlwriter.XMLwriter):
         self.is_chartsheet = 0
         self.page_view = 0
 
+        self.vba_codename = None
+
         self.date_1904 = False
         self.epoch = datetime.datetime(1899, 12, 31)
 
@@ -463,19 +466,19 @@ class Worksheet(xmlwriter.XMLwriter):
         return 0
 
     @convert_range_args
-    def write_array_formula(self, firstrow, firstcol, lastrow, lastcol,
+    def write_array_formula(self, first_row, first_col, last_row, last_col,
                             formula, cell_format=None, value=0):
         """
         Write a formula to a worksheet cell.
 
         Args:
-            firstrow:    The first row of the cell range. (zero indexed).
-            firstcol:    The first column of the cell range.
-            lastrow:     The last row of the cell range. (zero indexed).
-            lastcol:     The last column of the cell range.
-            formula:     Cell formula.
-            cell_format: An optional cell Format object.
-            value:       An optional value for the formula. Default is 0.
+            first_row:    The first row of the cell range. (zero indexed).
+            first_col:    The first column of the cell range.
+            last_row:     The last row of the cell range. (zero indexed).
+            last_col:     The last column of the cell range.
+            formula:      Cell formula.
+            cell_format:  An optional cell Format object.
+            value:        An optional value for the formula. Default is 0.
 
         Returns:
             0:  Success.
@@ -484,21 +487,21 @@ class Worksheet(xmlwriter.XMLwriter):
         """
 
         # Swap last row/col with first row/col as necessary.
-        if firstrow > lastrow:
-            firstrow, lastrow = lastrow, firstrow
-        if firstcol > lastcol:
-            firstcol, lastcol = lastcol, firstcol
+        if first_row > last_row:
+            first_row, last_row = last_row, first_row
+        if first_col > last_col:
+            first_col, last_col = last_col, first_col
 
         # Check that row and col are valid and store max and min values
-        if self._check_dimensions(lastrow, lastcol):
+        if self._check_dimensions(last_row, last_col):
             return -1
 
         # Define array range
-        if firstrow == lastrow and firstcol == lastcol:
-            cell_range = xl_rowcol_to_cell(firstrow, firstcol)
+        if first_row == last_row and first_col == last_col:
+            cell_range = xl_rowcol_to_cell(first_row, first_col)
         else:
-            cell_range = (xl_rowcol_to_cell(firstrow, firstcol) + ':'
-                          + xl_rowcol_to_cell(lastrow, lastcol))
+            cell_range = (xl_rowcol_to_cell(first_row, first_col) + ':'
+                          + xl_rowcol_to_cell(last_row, last_col))
 
         # Remove array formula braces and the leading =.
         if formula[0] == '{':
@@ -509,20 +512,20 @@ class Worksheet(xmlwriter.XMLwriter):
             formula = formula[:-1]
 
         # Write previous row if in in-line string optimization mode.
-        if self.optimization and firstrow > self.previous_row:
-            self._write_single_row(firstrow)
+        if self.optimization and first_row > self.previous_row:
+            self._write_single_row(first_row)
 
         # Store the cell data in the worksheet data table.
         cell_tuple = namedtuple('ArrayFormula',
                                 'formula, format, value, range')
-        self.table[firstrow][firstcol] = cell_tuple(formula, cell_format,
-                                                    value, cell_range)
+        self.table[first_row][first_col] = cell_tuple(formula, cell_format,
+                                                      value, cell_range)
 
         # Pad out the rest of the area with formatted zeroes.
         if not self.optimization:
-            for row in range(firstrow, lastrow + 1):
-                for col in range(firstcol, lastcol + 1):
-                    if row != firstrow or col != firstcol:
+            for row in range(first_row, last_row + 1):
+                for col in range(first_col, last_col + 1):
+                    if row != first_row or col != first_col:
                         self.write_number(row, col, 0, cell_format)
 
         return 0
@@ -845,7 +848,9 @@ class Worksheet(xmlwriter.XMLwriter):
 
         """
         if len(header) >= 255:
-            raise Exception('Header string must be less than 255 characters')
+            warnings.warn('Header string must be less than 255 characters',
+                          SyntaxWarning)
+            return
 
         self.header = header
         self.margin_header = margin
@@ -864,7 +869,9 @@ class Worksheet(xmlwriter.XMLwriter):
 
         """
         if len(footer) >= 255:
-            raise Exception('Footer string must be less than 255 characters')
+            warnings.warn('Footer string must be less than 255 characters',
+                          SyntaxWarning)
+            return
 
         self.footer = footer
         self.margin_footer = margin
@@ -961,6 +968,35 @@ class Worksheet(xmlwriter.XMLwriter):
         self.print_headers = 1
         self.print_options_changed = 1
 
+    @convert_range_args
+    def print_area(self, first_row, first_col, last_row, last_col):
+        """
+        Set the print area in the current worksheet.
+
+        Args:
+            first_row:    The first row of the cell range. (zero indexed).
+            first_col:    The first column of the cell range.
+            last_row:     The last row of the cell range. (zero indexed).
+            last_col:     The last column of the cell range.
+
+        Returns:
+            0:  Success.
+            -1: Row or column is out of worksheet bounds.
+
+        """
+        # Set the print area in the current worksheet.
+
+        # Ignore max print area since it is the same as no  area for Excel.
+        if (first_row == 0 and first_col == 0
+                and last_row == self.xls_rowmax - 1
+                and last_col == self.xls_colmax - 1):
+            return
+
+        # Build up the print area range "Sheet1!$A$1:$C$13".
+        area = self._convert_name_area(first_row, first_col,
+                                       last_row, last_col)
+        self.print_area_range = area
+
     def print_across(self):
         """
         Set the order in which pages are printed.
@@ -975,12 +1011,97 @@ class Worksheet(xmlwriter.XMLwriter):
         self.page_order = 1
         self.page_setup_changed = 1
 
+    def fit_to_pages(self, width, height):
+        """
+        Fit the printed area to a specific number of pages both vertically and
+        horizontally.
+
+        Args:
+            width:  Number of pages horizontally.
+            height: Number of pages vertically.
+
+        Returns:
+            Nothing.
+
+        """
+        self.fit_page = 1
+        self.fit_width = width
+        self.fit_height = height
+        self.page_setup_changed = 1
+
+    def set_start_page(self, start_page):
+        """
+        Set the start page number when printing.
+
+        Args:
+            start_page: Start page number.
+
+        Returns:
+            Nothing.
+
+        """
+        self.page_start = start_page
+        self.custom_start = 1
+
+    def set_print_scale(self, scale):
+        """
+        Set the scale factor for the printed page.
+
+        Args:
+            scale: Print scale. 10 <= scale <= 400.
+
+        Returns:
+            Nothing.
+
+        """
+        # Confine the scale to Excel's range.
+        if scale < 10 or scale > 400:
+            warnings.warn("Print scale outside range: 10 <= scale <= 400",
+                          SyntaxWarning)
+            return
+
+        # Turn off "fit to page" option when print scale is on.
+        self.fit_page = 0
+
+        self.print_scale = int(scale)
+        self.page_setup_changed = 1
+
+    def set_h_pagebreaks(self, breaks):
+        """
+        Set the horizontal page breaks on a worksheet.
+
+        Args:
+            breaks: List of rows where the page breaks should be added.
+
+        Returns:
+            Nothing.
+
+        """
+        self.hbreaks = breaks
+
+    #
+    # set_v_pagebreaks(@breaks)
+    #
+    # Store the vertical page breaks on a worksheet.
+    #
+    def set_v_pagebreaks(self, breaks):
+        """
+        Set the horizontal page breaks on a worksheet.
+
+        Args:
+            breaks: List of columns where the page breaks should be added.
+
+        Returns:
+            Nothing.
+
+        """
+        self.vbreaks = breaks
+
     ###########################################################################
     #
     # Private API.
     #
     ###########################################################################
-
     def _initialize(self, init_data):
         self.name = init_data['name']
         self.index = init_data['index']
@@ -997,7 +1118,7 @@ class Worksheet(xmlwriter.XMLwriter):
         self._write_worksheet()
 
         # Write the worksheet properties.
-        # self._write_sheet_pr()
+        self._write_sheet_pr()
 
         # Write the worksheet dimensions.
         self._write_dimension()
@@ -1055,10 +1176,10 @@ class Worksheet(xmlwriter.XMLwriter):
         self._write_header_footer()
 
         # Write the rowBreaks element.
-        # self._write_row_breaks()
+        self._write_row_breaks()
 
         # Write the colBreaks element.
-        # self._write_col_breaks()
+        self._write_col_breaks()
 
         # Write the drawing element.
         # self._write_drawings()
@@ -1187,6 +1308,72 @@ class Worksheet(xmlwriter.XMLwriter):
         else:
             return "'{}'".format(sheetname)
 
+    def _convert_name_area(self, row_num_1, col_num_1, row_num_2, col_num_2):
+        # Convert zero indexed rows and columns to the format required by
+        # worksheet named ranges, eg, "Sheet1!$A$1:$C$13".
+
+        range1 = ''
+        range2 = ''
+        area = ''
+        row_col_only = 0
+
+        # Convert to A1 notation.
+        col_char_1 = xl_col_to_name(col_num_1, 1)
+        col_char_2 = xl_col_to_name(col_num_2, 1)
+        row_char_1 = '$' + str(row_num_1 + 1)
+        row_char_2 = '$' + str(row_num_2 + 1)
+
+        # We need to handle special cases that refer to rows or columns only.
+        if row_num_1 == 0 and row_num_2 == self.xls_rowmax - 1:
+            range1 = col_char_1
+            range2 = col_char_2
+            row_col_only = 1
+        elif col_num_1 == 0 and col_num_2 == self.xls_colmax - 1:
+            range1 = row_char_1
+            range2 = row_char_2
+            row_col_only = 1
+        else:
+            range1 = col_char_1 + row_char_1
+            range2 = col_char_2 + row_char_2
+
+        # A repeated range is only written once (if it isn't a special case).
+        if range1 == range2 and not row_col_only:
+            area = range1
+        else:
+            area = range1 + ':' + range2
+
+        # Build up the print area range "Sheet1!$A$1:$C$13".
+        sheetname = self._quote_sheetname(self.name)
+        area = sheetname + "!" + area
+
+        return area
+
+    def _sort_pagebreaks(self, breaks):
+        # This is an internal method used to filter elements of a list of
+        # pagebreaks used in the _store_hbreak() and _store_vbreak() methods.
+        # It:
+        #   1. Removes duplicate entries from the list.
+        #   2. Sorts the list.
+        #   3. Removes 0 from the list if present.
+        if not breaks:
+            return
+
+        breaks_set = set(breaks)
+
+        if 0 in breaks_set:
+            breaks_set.remove(0)
+
+        breaks_list = list(breaks_set)
+        breaks_list.sort()
+
+        # The Excel 2007 specification says that the maximum number of page
+        # breaks is 1026. However, in practice it is actually 1023.
+        max_num_breaks = 1023
+        if len(breaks_list) > max_num_breaks:
+            breaks_list = breaks_list[:max_num_breaks]
+
+        return breaks_list
+
     ###########################################################################
     #
     # XML methods.
@@ -1449,6 +1636,10 @@ class Worksheet(xmlwriter.XMLwriter):
         else:
             attributes.append(('orientation', 'landscape'))
 
+        # Set start page for printing.
+        if self.page_start != 0:
+            attributes.append(('useFirstPageNumber', self.page_start))
+
         self._xml_empty_tag('pageSetup', attributes)
 
     def _write_print_options(self):
@@ -1768,3 +1959,121 @@ class Worksheet(xmlwriter.XMLwriter):
         ]
 
         self._xml_data_element('f', formula, attributes)
+
+    def _write_sheet_pr(self):
+        # Write the <sheetPr> element for Sheet level properties.
+        attributes = []
+
+        if (not self.fit_page
+                and not self.filter_on
+                and not self.tab_color
+                and not self.outline_changed
+                and not self.vba_codename):
+            return
+
+        if self.vba_codename:
+            attributes.append(('codeName', self.vba_codename))
+
+        if self.filter_on:
+            attributes.append(('filterMode', 1))
+
+        if (self.fit_page
+                or self.tab_color
+                or self.outline_changed):
+            self._xml_start_tag('sheetPr', attributes)
+            self._write_tab_color()
+            self._write_outline_pr()
+            self._write_page_set_up_pr()
+            self._xml_end_tag('sheetPr')
+        else:
+            self._xml_empty_tag('sheetPr', attributes)
+
+    def _write_page_set_up_pr(self):
+        # Write the <pageSetUpPr> element.
+        if not self.fit_page:
+            return
+
+        attributes = [('fitToPage', 1)]
+        self._xml_empty_tag('pageSetUpPr', attributes)
+
+    def _write_tab_color(self):
+        # Write the <tabColor> element.
+        color_index = self.tab_color
+
+        if not color_index:
+            return
+
+        rgb = self._get_palette_color(color_index)
+
+        attributes = [('rgb', rgb)]
+
+        self._xml_empty_tag('tabColor', attributes)
+
+    def _write_outline_pr(self):
+        # Write the <outlinePr> element.
+        attributes = []
+
+        if not self.outline_changed:
+            return
+
+        if self.outline_style:
+            attributes.append(("applyStyles", 1))
+        if not self.outline_below:
+            attributes.append(("summaryBelow", 0))
+        if not self.outline_right:
+            attributes.append(("summaryRight", 0))
+        if not self.outline_on:
+            attributes.append(("showOutlineSymbols", 0))
+
+        self._xml_empty_tag('outlinePr', attributes)
+
+    def _write_row_breaks(self):
+        # Write the <rowBreaks> element.
+        page_breaks = self._sort_pagebreaks(self.hbreaks)
+
+        if not page_breaks:
+            return
+
+        count = len(page_breaks)
+
+        attributes = [
+            ('count', count),
+            ('manualBreakCount', count),
+        ]
+
+        self._xml_start_tag('rowBreaks', attributes)
+
+        for row_num in (page_breaks):
+            self._write_brk(row_num, 16383)
+
+        self._xml_end_tag('rowBreaks')
+
+    def _write_col_breaks(self):
+        # Write the <colBreaks> element.
+        page_breaks = self._sort_pagebreaks(self.vbreaks)
+
+        if not page_breaks:
+            return
+
+        count = len(page_breaks)
+
+        attributes = [
+            ('count', count),
+            ('manualBreakCount', count),
+        ]
+
+        self._xml_start_tag('colBreaks', attributes)
+
+        for col_num in (page_breaks):
+            self._write_brk(col_num, 1048575)
+
+        self._xml_end_tag('colBreaks')
+
+    def _write_brk(self, brk_id, brk_max):
+        # Write the <brk> element.
+        attributes = [
+            ('id', brk_id),
+            ('max', brk_max),
+            ('man', 1)]
+
+        self._xml_empty_tag('brk', attributes)
